--- qubes.lua	2017-06-25 02:40:58.911986109 +0200
+++ qubes.lua	2017-06-25 02:58:51.992986109 +0200
@@ -193,6 +193,67 @@
     return menu
 end
 
+-- Return vmname of a client
+function qubes.get_vmname(c)
+	return client.property.get(c, 'qubes_vmname')
+end
+
+
+-- Return menu like qubes.make_menu() but with subs for Net/ProxyVMs and TemplateVMs
+function qubes.make_partitioned_menu()
+    local menu = {}
+    local menuproxy = {}
+    local menutemplate = {}
+    for line in io.popen([[python -c "
+import os.path
+import qubes.qubes
+qvmc = qubes.qubes.QubesVmCollection()
+qvmc.lock_db_for_reading()
+qvmc.load()
+qvmc.unlock_db()
+print('\n'.join('{} {} {}'.format(vm.name, vm.dir_path, vm.type)
+    for vm in sorted(qvmc.values(), key=lambda vm: vm.name)
+    if os.path.isdir(vm.dir_path)))
+"]]):lines() do
+        local vmname, vmpath, vmtype = line:match('^(.+) (.+) (.+)$')
+        io.stderr:write('line=' .. line .. '\n')
+        if vmtype == "TemplateVM" then
+            table.insert(menutemplate, qubes.make_vm_menu(vmname, vmpath))
+        elseif vmtype == "ProxyVM" or vmtype == "NetVM" then
+            table.insert(menuproxy, qubes.make_vm_menu(vmname, vmpath))
+	else
+            table.insert(menu, qubes.make_vm_menu(vmname, vmpath))
+	end
+    end
+
+    table.insert(menu, {'Template VMs', menutemplate,
+        '/usr/share/icons/hicolor/16x16/apps/qubes-logo-icon.png'})
+
+    table.insert(menu, {'Proxy/Net VMs', menuproxy,
+        '/usr/share/icons/hicolor/16x16/apps/qubes-logo-icon.png'})
+
+    table.insert(menu, {'Qubes Manager', 'qubes-manager',
+        '/usr/share/icons/hicolor/16x16/apps/qubes-manager.png'})
+    return menu
+end
+
+-- Execute command in a vm, including dom0
+function qubes.execute_in_vm(vmname, command)
+    if vmname == "dom0" then
+        util.spawn(command)
+    else
+        util.spawn("qvm-run -a " .. vmname .. " ".. command)
+    end
+end
+
+-- Execute command in the vm of the client, including dom0
+function qubes.execute_in_vm_of_client(c, command)
+    local vmname = qubes.get_vmname(c)
+    if vmname ~= nil then
+        qubes.execute_in_vm(vmname, command)
+    end
+end
+
 return qubes
 
 -- vim: ts=4 sw=4 et
